import sys

ACGT = ( "A", "C", "G", "T" )
ACGTN = ( "A", "C", "G", "T", "N" )
ACGTNOther = ( "A", "C", "G", "T", "N", "Other")

Dinucs = []
CompDinucs = {}

def D(Dinuc, Comp):
	Dinucs.append(Dinuc)
	CompDinucs[Dinuc] = Comp

D("AA", "TT")
D("AC", "GT")
D("AG", "CT")
D("AT", "AT")

D("CA", "TG")
D("CC", "GG")
D("CG", "CG")
# D("CT", "AG")

D("GA", "TC")
D("GC", "GC")
# D("GG", "CC")
# D("GT", "AC")

D("TA", "TA")
# D("TC", "GA")
# D("TG", "CA")
# D("TT", "AA")

def GetPct(i, N):
	if N == 0:
		return 0
	return 100.0*float(i)/float(N)

def GetCounts(FileName):
	One = {}
	Two = {}

	File = open(FileName)
	Lastc = ""
	while 1:
		Line = File.readline()
		if len(Line) == 0:
			break
		if Line[0] == ">":
			continue
		Line = Line.upper()
		for c in Line:
			if c.isspace():
				continue
			c = c.upper()
			if c not in One.keys():
				One[c] = 0
			One[c] += 1
			if Lastc != "":
				Dinuc = Lastc + c
				if Dinuc not in Two.keys():
					Two[Dinuc] = 0
				Two[Dinuc] += 1
			Lastc = c
	return One, Two

One, Two = GetCounts(sys.argv[1])

Bases = 0
for c in One.keys():
	if c in ACGT:
		Bases += One[c]

print ""

print "Letter       Bases       Pct"
print "======  ==========  ========"
for c in [ "A", "T", "C", "G" ]:
	Pct = GetPct(One[c], Bases)
	if c in ACGT:
		print "%6s  %10u   %6.1f%%" % (c, One[c], Pct)
	if c == "T":
		print ""

Any = 0
for c in One.keys():
	Pct = GetPct(One[c], Bases)
	if c not in ACGT:
		if not Any:
			print ""
		Any = 1
		print "%6s  %10u" % (c, One[c])

print ""
ATBases = One["A"] + One["T"]
CGBases = One["G"] + One["C"]

ATPct = GetPct(ATBases, Bases)
CGPct = GetPct(CGBases, Bases)

print "   A+T  %10u   %6.1f%%" % (ATBases, ATPct)
print "   G+C  %10u   %6.1f%%" % (CGBases, CGPct)
print ""
print " Total  %10u   %6.1f%%" % (ATBases + CGBases, ATPct + CGPct)
print ""
print ""

if Bases < 2:
	sys.exit(0)

Total = 0
for Dinuc in Dinucs:
	Comp = CompDinucs[Dinuc]
	Count = Two[Dinuc]
	CompCount = Two[Comp]
	if Comp == Dinuc:
		Total += Count
	else:
		Total += Count + CompCount
	
print " Dinuc    Comp          Nr     Comp Nr     Diff      Pct"
print "======  ======  ==========  ==========  =======  ======="
SumPct = 0.0
for Dinuc in Dinucs:
	Comp = CompDinucs[Dinuc]
	Count = Two[Dinuc]
	CompCount = Two[Comp]
	DiffPct = GetPct(CompCount - Count, Count)
	if Comp == Dinuc:
		Pct = GetPct(Count, Bases - 1)
		SumPct += Pct
		print "%6s     =%2s  %10u                       %6.1f%%" % (Dinuc, Dinuc, Count, Pct)
	else:
		Pct = GetPct(Count + CompCount, Bases - 1)
		SumPct += Pct
		print "%6s  %6s  %10u  %10u  %6.1f%%  %6.1f%%" % (Dinuc, Comp, Count, CompCount, DiffPct, Pct)

print ""
print " Total          %10u                       %6.1f%%" % (Total, SumPct)

print ""
print ""

TotalCs = Two["CA"] + Two["CC"] + Two["CG"] + Two["CT"]
TotalGs = Two["AG"] + Two["CG"] + Two["GG"] + Two["TG"]

print "%10u  Total Cs in C{ACGT} dinucleotides" % TotalCs
print "%10u  Cs in CpGs = %.1f%%" % (Two["CG"], GetPct(Two["CG"], TotalCs))

print ""
print "%10u  Total Gs in {ACGT}G dinucleotides" % TotalGs
print "%10u  Gs in CpGs = %.1f%%" % (Two["CG"], GetPct(Two["CG"], TotalGs))
